unit ViewTreeForm;
{$ifdef FPC}{$mode objfpc}{$h+}{$endif}
interface
uses
 msetypes,mseglob,mseguiglob,mseguiintf,mseapplication,msestat,msemenus,msegui,
 msegraphics,msegraphutils,mseevent,mseclasses,msewidgets,mseforms,
 msesimplewidgets,mseedit,msestatfile,msestream,msestrings,sysutils,mseact,
 msedataedits,msegrids,mseificomp,mseificompglob,mseifiglob,msewidgetgrid,
 msedatanodes,mselistbrowser;
 
type

  { TfrmViewTree }

  TfrmViewTree = class(tsubform)
    tsimplewidget3: tsimplewidget;
    btnAdd: tbutton;
    btnView: tbutton;
    tedit1: tedit;
    twidgetgrid1: twidgetgrid;
    ttreeitemedit1: ttreeitemedit;
   procedure Expand(const sender: TObject);
   procedure PrintSelected(const sender: TObject);
  private
    FLibraryNode: ttreelistedititem;
    FToExpand: ttreelistedititem;
    procedure LoadDirTreeView(const sender: tobject);
  protected
    procedure doloaded; override;
  end;
 
implementation

uses
 ViewTreeForm_mfm, msedrawtext;

type

  { TGenericNode }

  TGenericNode = class(ttreelistedititem)
  private
    FExtraText: String;
  public
    property ExtraText: String read FExtraText write FExtraText;
    procedure updatecaption(const acanvas: tcanvas;
      var alayoutinfo: listitemlayoutinfoty; var ainfo: drawtextinfoty); override;
    procedure drawimage(const acanvas: tcanvas;
      var alayoutinfo: listitemlayoutinfoty);override;
  end;

  { TSongNode }

  TSongNode = class(TGenericNode)
  private
    FId: Int64;
  public
    constructor create(const aowner: tcustomitemlist = nil;
               const aparent: ttreelistitem = nil); override;
  end;

  { TBoldNode }

  TBoldNode = class(TGenericNode)
  protected
    ffont: tfont;
  public
    constructor create(const aowner: tcustomitemlist = nil;
               const aparent: ttreelistitem = nil); override;
    destructor destroy; override;
    procedure updatecaption(const acanvas: tcanvas;
      var alayoutinfo: listitemlayoutinfoty; var ainfo: drawtextinfoty); override;
  end;

  { TDirNode }

  TDirNode = class(TBoldNode)
  protected
    procedure setexpanded(const Value: boolean);override;
  public
    constructor create(const aowner: tcustomitemlist = nil;
      const aparent: ttreelistitem = nil); override;
  end;

  { TArtistNode }

  TArtistNode = class(TBoldNode)
  public
    constructor create(const aowner: tcustomitemlist = nil;
      const aparent: ttreelistitem = nil); override;
  end;

{ TArtistNode }

constructor TArtistNode.create(const aowner: tcustomitemlist;
  const aparent: ttreelistitem);
begin
  inherited create(aowner, aparent);

  //imagenr := iICO16_USER_O;
end;

constructor TSongNode.create(const aowner: tcustomitemlist;
  const aparent: ttreelistitem);
begin
  inherited create(aowner, aparent);

  imagenr := -1;
  FId := -1;
end;

procedure TDirNode.setexpanded(const Value: boolean);
begin
  if Value
  then imagenr := 0
  else imagenr := 0;
  inherited setexpanded(Value);
end;

constructor TDirNode.create(const aowner: tcustomitemlist;
  const aparent: ttreelistitem);
begin
  inherited create(aowner, aparent);

  imagenr := 0;
end;

{ TGenericNode }

procedure TGenericNode.updatecaption(const acanvas: tcanvas;
  var alayoutinfo: listitemlayoutinfoty; var ainfo: drawtextinfoty);
var w: int32;
begin
  inherited updatecaption(acanvas, alayoutinfo, ainfo);

  w:= acanvas.getstringwidth(FExtraText);
  dec(ainfo.dest.cx,w);
  dec(ainfo.clip.cx,w);
end;

procedure TGenericNode.drawimage(const acanvas: tcanvas;
  var alayoutinfo: listitemlayoutinfoty);
begin
  inherited drawimage(acanvas, alayoutinfo);

  if not alayoutinfo.variable.calcautocellsize then
    with alayoutinfo do
    begin
      dec(captioninnerrect.cx,variable.treelevelshift);
      dec(captionrect.cx,variable.treelevelshift);
      drawtext(acanvas,FExtraText,captionrect,captioninnerrect,
        [tf_clipo,tf_ycentered,tf_right,tf_grayed]);
      inc(captioninnerrect.cx,variable.treelevelshift);
      inc(captionrect.cx,variable.treelevelshift);
    end;
end;

{ TBoldNode }

constructor TBoldNode.create(const aowner: tcustomitemlist;
  const aparent: ttreelistitem);
begin
  ffont:= tfont.create();
  ffont.bold:= true;
  inherited create(aowner, aparent);
end;

destructor TBoldNode.destroy;
begin
  inherited destroy;
  ffont.free;
end;

procedure TBoldNode.updatecaption(const acanvas: tcanvas;
  var alayoutinfo: listitemlayoutinfoty; var ainfo: drawtextinfoty);
begin
  inherited updatecaption(acanvas, alayoutinfo, ainfo);
  ainfo.font:= ffont;
end;

{ TfrmViewTree }

procedure TfrmViewTree.doloaded;
begin
  inherited doloaded;

  FLibraryNode := TBoldNode.create();
  FLibraryNode.caption := 'Library';
  ttreeitemedit1.itemlist.add(FLibraryNode);

  LoadDirTreeView(nil);
end;

procedure TfrmViewTree.LoadDirTreeView(const sender: tobject);
var 
  d1, d2: TDirNode;
  s: TSongNode;
begin
  twidgetgrid1.beginupdate;
  ttreeitemedit1.beginupdate;
  try
    FLibraryNode.clear;
    
    d1 := TDirNode.Create;
    d1.Caption := 'mp32';
    d1.FExtraText := '2';
    FLibraryNode.add(d1);
    
    d2 := TDirNode.Create;
    d2.Caption := '2006 A Good Year';
    d2.FExtraText := '2';
    d1.add(d2);
    s := TSongNode.Create;
    s.Caption := 'Soundtrack - How Can I Be Sure Of You';
    s.FExtraText := '03:05';
    d2.add(s);
    FToExpand := s;
    s := TSongNode.Create;
    s.Caption := 'Il Faut Du Temps Au Temps';
    s.FExtraText := '02:21';
    d2.add(s);
    
    d2 := TDirNode.Create;
    d2.Caption := 'adskip';
    d1.add(d2);
    TGenericNode(FLibraryNode).FExtraText := IntToStr(FLibraryNode.count);
  finally
    ttreeitemedit1.endupdate;
    twidgetgrid1.endupdate(True);
  end;
end;

procedure TfrmViewTree.Expand(const sender: TObject);
begin
  if FToExpand<>nil then
    FToExpand.expandtoroot;
end;

procedure TfrmViewTree.PrintSelected(const sender: TObject);
begin
  writeln(length(ttreeitemedit1.selecteditems),'|',Length(FLibraryNode.getselectednodes()));
end;

end.
